@@ -44,8 +44,7 @@ func (e *Exporter) getKeyInfo(ch chan<- prometheus.Metric, c redis.Conn, dbLabel
 
 	switch keyType {
 	case "none":
-		log.Debugf("Key '%s' not found when trying to get type and size: using default '0.0'", keyName)
-		e.registerConstMetricGauge(ch, "key_size", 0.0, dbLabel, keyName)
+		log.Debugf("Key '%s' not found, skipping", keyName)
 		return
 
 	case "string":
@@ -83,18 +82,17 @@ func (e *Exporter) getKeyInfo(ch chan<- prometheus.Metric, c redis.Conn, dbLabel
 	}
 }
 
-func (e *Exporter) extractCheckKeyMetrics(ch chan<- prometheus.Metric, c redis.Conn) {
+func (e *Exporter) extractCheckKeyMetrics(ch chan<- prometheus.Metric, c redis.Conn) error {
+
 	keys, err := parseKeyArg(e.options.CheckKeys)
 	if err != nil {
-		log.Errorf("Couldn't parse check-keys: %#v", err)
-		return
+		return fmt.Errorf("couldn't parse check-keys: %w", err)
 	}
 	log.Debugf("keys: %#v", keys)
 
 	singleKeys, err := parseKeyArg(e.options.CheckSingleKeys)
 	if err != nil {
-		log.Errorf("Couldn't parse check-single-keys: %#v", err)
-		return
+		return fmt.Errorf("couldn't parse check-single-keys: %w", err)
 	}
 	log.Debugf("e.singleKeys: %#v", singleKeys)
 
@@ -119,6 +117,7 @@ func (e *Exporter) extractCheckKeyMetrics(ch chan<- prometheus.Metric, c redis.C
 	} else {
 		e.extractCheckKeyMetricsPipelined(ch, c, allKeys)
 	}
+	return nil
 }
 
 func (e *Exporter) extractCheckKeyMetricsPipelined(ch chan<- prometheus.Metric, c redis.Conn, allKeys []dbKeyPair) {
@@ -143,7 +142,6 @@ func (e *Exporter) extractCheckKeyMetricsPipelined(ch chan<- prometheus.Metric,
 			keysByDb[k.db] = []string{k.key}
 		}
 	}
-	log.Debugf("keysByDb: %#v", keysByDb)
 
 	for dbNum, arrayOfKeys := range keysByDb {
 		dbLabel := "db" + dbNum
@@ -197,7 +195,7 @@ func (e *Exporter) extractCheckKeyMetricsPipelined(ch chan<- prometheus.Metric,
 			}
 			memUsageInBytes, err := redis.Int64(c.Receive())
 			if err != nil {
-				log.Errorf("key: [%s] - memUsageInBytes Receive() err: %s", keyName, err)
+				// log.Errorf("key: [%s] - memUsageInBytes Receive() err: %s", keyName, err)
 				continue
 			}
 
@@ -232,13 +230,13 @@ func (e *Exporter) getKeyInfoPipelined(ch chan<- prometheus.Metric, c redis.Conn
 
 			log.Debugf("c.Send() STRLEN  args: [%v]", keyName)
 			if err := c.Send("STRLEN", keyName); err != nil {
-				log.Errorf("PFCOUNT err: %s", err)
+				log.Errorf("STRLEN err: %s", err)
 				return
 			}
 
 			log.Debugf("c.Send() GET  args: [%v]", keyName)
 			if err := c.Send("GET", keyName); err != nil {
-				log.Errorf("PFCOUNT err: %s", err)
+				log.Errorf("GET err: %s", err)
 				return
 			}
 
@@ -296,9 +294,7 @@ func (e *Exporter) getKeyInfoPipelined(ch chan<- prometheus.Metric, c redis.Conn
 
 		switch keyType {
 		case "none":
-			log.Debugf("Key '%s' not found when trying to get type and size: using default '0.0'", keyName)
-			e.registerConstMetricGauge(ch, "key_size", 0.0, dbLabel, keyName)
-			return
+			log.Debugf("Key '%s' not found, skipping", keyName)
 
 		case "string":
 			hllSize, hllErr := redis.Int64(c.Receive())
@@ -457,7 +453,7 @@ func parseKeyArg(keysArgString string) (keys []dbKeyPair, err error) {
 			db = "0"
 			key, err = url.QueryUnescape(strings.TrimSpace(frags[0]))
 		case 2:
-			db = strings.Replace(strings.TrimSpace(frags[0]), "db", "", -1)
+			db = strings.ReplaceAll(strings.TrimSpace(frags[0]), "db", "")
 			key, err = url.QueryUnescape(strings.TrimSpace(frags[1]))
 		default:
 			return keys, fmt.Errorf("invalid key list argument: %s", k)
@@ -475,7 +471,7 @@ func parseKeyArg(keysArgString string) (keys []dbKeyPair, err error) {
 
 		number, err := strconv.Atoi(db)
 		if err != nil || number < 0 {
-			return keys, fmt.Errorf("Invalid database index for db \"%s\": %s", db, err)
+			return keys, fmt.Errorf("invalid database index for db \"%s\": %s", db, err)
 		}
 
 		keys = append(keys, dbKeyPair{db, key})
@@ -487,7 +483,7 @@ func parseKeyArg(keysArgString string) (keys []dbKeyPair, err error) {
 // This function was adapted from: https://github.com/reisinger/examples-redigo
 func scanKeys(c redis.Conn, pattern string, count int64) (keys []interface{}, err error) {
 	if pattern == "" {
-		return keys, fmt.Errorf("Pattern shouldn't be empty")
+		return keys, fmt.Errorf("pattern shouldn't be empty")
 	}
 
 	iter := 0
