@@ -1,15 +1,11 @@
 package exporter
 
 import (
-	"bytes"
 	"net/http/httptest"
 	"net/url"
 	"os"
 	"strings"
 	"testing"
-
-	"github.com/prometheus/client_golang/prometheus"
-	log "github.com/sirupsen/logrus"
 )
 
 func TestHostVariations(t *testing.T) {
@@ -87,19 +83,12 @@ func TestPasswordProtectedInstance(t *testing.T) {
 				tst.addr,
 				Options{
 					Namespace: "test",
-					Registry:  prometheus.NewRegistry(),
 					User:      tst.user,
 					Password:  tst.pwd,
 				})
 			ts := httptest.NewServer(e)
 			defer ts.Close()
 
-			chM := make(chan prometheus.Metric, 10000)
-			go func() {
-				e.Collect(chM)
-				close(chM)
-			}()
-
 			body := downloadURL(t, ts.URL+"/metrics")
 			if !strings.Contains(body, "test_up 1") {
 				t.Errorf(`%s - response to /metric doesn't contain "test_up 1"`, tst)
@@ -116,16 +105,10 @@ func TestPasswordInvalid(t *testing.T) {
 	testPwd := "redis-password"
 	uri := strings.Replace(os.Getenv("TEST_PWD_REDIS_URI"), testPwd, "wrong-pwd", -1)
 
-	e, _ := NewRedisExporter(uri, Options{Namespace: "test", Registry: prometheus.NewRegistry()})
+	e, _ := NewRedisExporter(uri, Options{Namespace: "test"})
 	ts := httptest.NewServer(e)
 	defer ts.Close()
 
-	chM := make(chan prometheus.Metric, 10000)
-	go func() {
-		e.Collect(chM)
-		close(chM)
-	}()
-
 	want := `test_exporter_last_scrape_error{err="dial redis: unknown network redis"} 1`
 	body := downloadURL(t, ts.URL+"/metrics")
 	if !strings.Contains(body, want) {
@@ -134,9 +117,9 @@ func TestPasswordInvalid(t *testing.T) {
 }
 
 func TestConnectToClusterUsingPasswordFile(t *testing.T) {
-	clusterUri := os.Getenv("TEST_REDIS_CLUSTER_PASSWORD_URI")
+	clusterUri := os.Getenv("TEST_VALKEY_CLUSTER_PASSWORD_URI")
 	if clusterUri == "" {
-		t.Skipf("TEST_REDIS_CLUSTER_PASSWORD_URI is not set")
+		t.Skipf("TEST_VALKEY_CLUSTER_PASSWORD_URI is not set")
 	}
 	passMap := map[string]string{clusterUri: "redis-password"}
 	wrongPassMap := map[string]string{"redis://redis-cluster-password-wrong:7006": "redis-password"}
@@ -158,21 +141,14 @@ func TestConnectToClusterUsingPasswordFile(t *testing.T) {
 				PasswordMap:         tst.passMap,
 				IsCluster:           tst.isCluster,
 			})
-			var buf bytes.Buffer
-			log.SetOutput(&buf)
-			defer func() {
-				log.SetOutput(os.Stderr)
-			}()
 			_, err := e.connectToRedisCluster()
 			t.Logf("connectToRedisCluster() err: %s", err)
-			if strings.Contains(buf.String(), "Cluster refresh failed:") && !tst.refreshError {
-				t.Errorf("Test Cluster connection Failed error")
+			if err != nil && strings.Contains(err.Error(), "Cluster refresh failed:") && !tst.refreshError {
+				t.Fatalf("Test Cluster connection Failed error")
 			}
-			if err != nil {
-				t.Errorf("Test Cluster connection Failed-connection error")
+			if !tst.refreshError && err != nil {
+				t.Fatalf("Test Cluster connection Failed, err: %s", err)
 			}
-
 		})
 	}
-
 }
