@@ -1,6 +1,7 @@
 package exporter
 
 import (
+	"net/http/httptest"
 	"os"
 	"strings"
 	"testing"
@@ -56,7 +57,7 @@ func TestStreamsGetStreamInfo(t *testing.T) {
 	tsts := []scanStreamFixture{
 		{
 			name:   "Stream test",
-			stream: TestKeysStreamName,
+			stream: TestKeyNameStream,
 			streamInfo: streamInfo{
 				Length:         2,
 				RadixTreeKeys:  1,
@@ -123,7 +124,7 @@ func TestStreamsGetStreamInfoUsingValKey7(t *testing.T) {
 	tsts := []scanStreamFixture{
 		{
 			name:   "Stream test",
-			stream: TestKeysStreamName,
+			stream: TestKeyNameStream,
 			streamInfo: streamInfo{
 				Length:         2,
 				RadixTreeKeys:  1,
@@ -496,7 +497,7 @@ func TestStreamsExtractStreamMetrics(t *testing.T) {
 	addr := os.Getenv("TEST_REDIS_URI")
 	e, _ := NewRedisExporter(
 		addr,
-		Options{Namespace: "test", CheckSingleStreams: dbNumStrFull + "=" + TestKeysStreamName},
+		Options{Namespace: "test", CheckSingleStreams: dbNumStrFull + "=" + TestKeyNameStream},
 	)
 	c, err := redis.DialURL(addr)
 	if err != nil {
@@ -554,7 +555,7 @@ func TestStreamsExtractStreamMetricsExcludeConsumer(t *testing.T) {
 	addr := os.Getenv("TEST_REDIS_URI")
 	e, _ := NewRedisExporter(
 		addr,
-		Options{Namespace: "test", CheckSingleStreams: dbNumStrFull + "=" + TestKeysStreamName, StreamsExcludeConsumerMetrics: true},
+		Options{Namespace: "test", CheckSingleStreams: dbNumStrFull + "=" + TestKeyNameStream, StreamsExcludeConsumerMetrics: true},
 	)
 	c, err := redis.DialURL(addr)
 	if err != nil {
@@ -620,3 +621,133 @@ func TestStreamsExtractStreamMetricsExcludeConsumer(t *testing.T) {
 		}
 	}
 }
+
+func TestClusterStreamMetricsExtraction(t *testing.T) {
+	if os.Getenv("TEST_REDIS_CLUSTER_MASTER_URI") == "" {
+		t.Skipf("TEST_REDIS_CLUSTER_MASTER_URI not set - skipping cluster stream test")
+	}
+
+	clusterURI := os.Getenv("TEST_REDIS_CLUSTER_MASTER_URI")
+
+	// Test streams to create
+	testStreams := []string{"audit_stream", "sa_audit_stream", "test_stream_cluster"}
+
+	// Setup cluster connection to create test streams
+	// Use cluster-aware connection to avoid MOVED errors
+	tempExporter, err := NewRedisExporter(clusterURI, Options{IsCluster: true})
+	if err != nil {
+		t.Fatalf("Couldn't create temp exporter for cluster setup: %v", err)
+	}
+
+	c, err := tempExporter.connectToRedisCluster()
+	if err != nil {
+		t.Fatalf("Couldn't connect to cluster: %v", err)
+	}
+	defer c.Close()
+
+	// Create test streams with some data using cluster connection
+	for _, streamName := range testStreams {
+		// Add entries to streams - cluster connection handles MOVED redirects automatically
+		_, err = c.Do("XADD", streamName, "*", "field1", "value1", "field2", "value2")
+		if err != nil {
+			t.Logf("Warning: couldn't create stream %s: %v", streamName, err)
+			continue
+		}
+		_, err = c.Do("XADD", streamName, "*", "field3", "value3")
+		if err != nil {
+			t.Logf("Warning: couldn't add to stream %s: %v", streamName, err)
+		}
+	}
+
+	// Cleanup function - use the same cluster connection to avoid MOVED errors
+	defer func() {
+		for _, streamName := range testStreams {
+			_, err := c.Do("DEL", streamName)
+			if err != nil {
+				t.Logf("Warning: couldn't clean up stream %s: %v", streamName, err)
+			}
+		}
+	}()
+
+	// Create exporter with cluster mode and single streams config
+	// This reproduces the exact command from the GitHub issue:
+	// redis_exporter --check-single-streams=audit,sa_audit --is-cluster=true
+	streamConfig := "db0=audit_stream,db0=sa_audit_stream,db0=test_stream_cluster"
+
+	e, err := NewRedisExporter(
+		clusterURI,
+		Options{
+			Namespace:          "test",
+			CheckSingleStreams: streamConfig,
+			IsCluster:          true,
+		},
+	)
+	if err != nil {
+		t.Fatalf("NewRedisExporter() err: %s", err)
+	}
+
+	// Test the full HTTP endpoint (this tests the complete path including cluster connection fix)
+	ts := httptest.NewServer(e)
+	defer ts.Close()
+
+	metricsOutput := downloadURL(t, ts.URL+"/metrics")
+
+	// Check if we got HTML instead of metrics (indicates an error during metrics collection)
+	if strings.Contains(metricsOutput, "<html>") {
+		t.Logf("Got HTML response instead of metrics, this indicates an error during metrics collection")
+		t.Logf("This could be due to Redis connection issues or cluster MOVED errors")
+		t.Logf("First 500 chars of response: %.500s...", metricsOutput)
+		t.Fatal("Expected Prometheus metrics but got HTML error page - check Redis cluster connectivity")
+	}
+
+	// Parse the metrics output to find stream metrics
+	foundMetrics := make(map[string]bool)
+	lines := strings.Split(metricsOutput, "\n")
+
+	for _, line := range lines {
+		// Look for stream_length metrics with our test streams
+		if strings.Contains(line, "stream_length") {
+			for _, streamName := range testStreams {
+				if strings.Contains(line, `stream="`+streamName+`"`) {
+					foundMetrics[streamName] = true
+					t.Logf("Found stream metric for: %s", streamName)
+				}
+			}
+		}
+	}
+
+	// Verify that we found metrics for our test streams
+	// This ensures that the cluster MOVED errors are properly handled
+	expectedStreams := 0
+	for _, streamName := range testStreams {
+		if foundMetrics[streamName] {
+			expectedStreams++
+			t.Logf("✓ Successfully found metrics for stream: %s", streamName)
+		} else {
+			t.Logf("⚠ Did not find metrics for stream: %s", streamName)
+		}
+	}
+
+	if expectedStreams == 0 {
+		t.Error("Expected to find metrics for at least one test stream in cluster mode")
+		t.Error("This indicates that the cluster MOVED error fix may not be working properly")
+
+		// Additional debugging info
+		t.Logf("Test streams created: %v", testStreams)
+		t.Logf("Found stream metrics for: %v", foundMetrics)
+
+		// Show sample of metrics output for debugging
+		sampleLines := strings.Split(metricsOutput, "\n")
+		t.Log("Sample metrics output (first 10 lines):")
+		for i, line := range sampleLines {
+			if i >= 10 || line == "" {
+				break
+			}
+			t.Logf("  %s", line)
+		}
+	} else {
+		t.Logf("✓ SUCCESS: Found stream metrics for %d/%d streams in cluster mode", expectedStreams, len(testStreams))
+		t.Logf("This confirms that Redis cluster MOVED errors are properly handled for streams")
+		t.Logf("HTTP endpoint successfully returned metrics without cluster MOVED errors")
+	}
+}
